#include <Screen.h>
#include <MenuItem.h>
#include <Directory.h>
#include <Entry.h>
#include <Alert.h>
#include <Path.h>
#include <Message.h>
#include <Roster.h>
#include <stdio.h>
#include <Bitmap.h>
#include <NodeInfo.h>
#include "BarneyBarWindow.h"
#include "BarneyBarMenu.h"

BarneyBarWindow::BarneyBarWindow():
	BWindow(BRect(0,0,0,0),"BarneyBar", B_NO_BORDER_WINDOW_LOOK, 
	B_FLOATING_ALL_WINDOW_FEEL, B_WILL_ACCEPT_FIRST_CLICK | 
	B_ASYNCHRONOUS_CONTROLS )
{
	BScreen* myScreen;
	display_mode mode;
	BMenuItem *MenuItem;
	BarneyBarMenu *BarneyMenu;
	BDirectory *Directory, *Directory2;
	BEntry *Entry, *Entry2;
	BMenu *Menu;
	BMessage *Message;
	entry_ref entry;
	char temp[80], temp2[80];
	BNodeInfo *NodeInfo;
	BNode *Node;
	BBitmap *icon = new BBitmap(BRect(0,0,15,15),B_COLOR_8_BIT);
	
	myScreen = new BScreen( B_MAIN_SCREEN_ID );
	myScreen->GetMode( &mode );

	ResizeTo( mode.virtual_width-1, 18 );		
	MenuBar = new BMenuBar( BRect(8,0,Bounds().right,Bounds().bottom),
		"MenuBar", B_FOLLOW_TOP | B_FOLLOW_BOTTOM, B_ITEMS_IN_ROW, false );
	//MenuBar->SetFont(be_plain_font);
	AddChild( MenuBar );
	
	SetPulseRate(1000000);
	
	// Populate MenuBar...
	Directory = new BDirectory("/boot/home/config/BarneyBar");
	Entry = new BEntry();	

	BeMenuMenu = new BMenu("BeMenu");
	BeMenuMenu->SetFont(be_plain_font);
	BeMenu = new BarneyBarBeMenu( BeMenuMenu );
	MenuBar->AddItem( BeMenu );	
		
	while ( Directory->GetNextEntry( Entry ) == B_OK )
	{
		if ( Entry->IsDirectory() == true )
		{
		Entry->GetName(temp);
			
		// Create and populate sub menu..	
		sprintf( temp2, "/boot/home/config/BarneyBar/%s", temp );
		Menu = new BMenu(temp);
		Directory2 = new BDirectory(temp2);
		Entry2 = new BEntry();
		while ( Directory2->GetNextEntry( Entry2 ) == B_OK )
		{
			// resolve link..
			while ( Entry2->IsSymLink() == true )
			{
				Entry2->GetRef( &entry );
				Entry2->SetTo( &entry, true );
			}
			
			Node = new BNode( Entry2 );
			NodeInfo = new BNodeInfo( Node );
			icon = new BBitmap(BRect(0,0,15,15),B_COLOR_8_BIT);
			NodeInfo->GetTrackerIcon(icon, B_MINI_ICON);

			Entry2->GetName(temp);
			Message = new BMessage( BARNEYBAR_MENUSELECTION );
			Entry2->GetRef( &entry );
			Message->AddRef("path", &entry );
			BarneyMenu = new BarneyBarMenu( temp, Message, icon );
			Menu->AddItem( BarneyMenu );		
		}
		Menu->SetFont(be_plain_font);
		MenuItem = new BMenuItem( Menu, NULL );
		MenuBar->AddItem(MenuItem);
		}
	}
	
	Clock = new BarneyBarClock(Bounds().right-66);
	MenuBar->AddChild( Clock );
	
	Pager = new BarneyBarPager(Bounds().right-(count_workspaces()*16)-82);
	MenuBar->AddChild( Pager );
	
	Divider= new BarneyBarDivider( Bounds().right-76 );
	MenuBar->AddChild( Divider );
	
	LeftEnd = new BarneyBarEndView( 0, true );
	AddChild( LeftEnd );
	RightEnd = new BarneyBarEndView( Bounds().right-15, false );
	MenuBar->AddChild( RightEnd );	
	LeftCap = new BarneyBarLeftCap();
	MenuBar->AddChild( LeftCap );
	
	// Populate BeMenu
	MenuItem = new BMenuItem( "About..", (new BMessage(BARNEYBAR_BEMENU_ABOUT) ));
	BeMenuMenu->AddItem( MenuItem );
	BeMenuMenu->AddSeparatorItem();
	MenuItem = new BMenuItem( "Find..",  (new BMessage(BARNEYBAR_BEMENU_FIND) ));
	BeMenuMenu->AddItem( MenuItem );	
	BeMenuMenu->AddSeparatorItem();
	MenuItem = new BMenuItem( "Preferences",  (new BMessage(BARNEYBAR_BEMENU_PREFS) ));
	BeMenuMenu->AddItem( MenuItem );	
	BeMenuMenu->AddSeparatorItem();	
	MenuItem = new BMenuItem( "Restart",  (new BMessage(BARNEYBAR_BEMENU_REBOOT) ));
	BeMenuMenu->AddItem( MenuItem );
	MenuItem = new BMenuItem( "Shutdown",  (new BMessage(BARNEYBAR_BEMENU_SHUTDOWN) ));
	BeMenuMenu->AddItem( MenuItem );
}

void BarneyBarWindow::MessageReceived( BMessage *message )
{
	BMessenger *messenger;
	entry_ref entry;
	BEntry *Entry;
	char *temp[1];
	BPath *Path;
	
	switch ( message->what )
	{
		case BARNEYBAR_MENUSELECTION:
			message->FindRef("path", &entry);
			Entry = new BEntry( &entry );
						
			while ( Entry->IsSymLink() == true )
			{
				Entry->GetRef( &entry );
				Entry->SetTo( &entry, true );
			}

			if ( Entry->IsDirectory() == true )
			{
				Path = new BPath();
				Entry->GetPath(Path); 
				temp[0]=(char*)Path->Path();
				be_roster->Launch( "application/x-vnd.Be-TRAK", 1, temp );
			}
			
			if ( Entry->IsFile() == true )
				be_roster->Launch( &entry );
			
			break;
		case BARNEYBAR_BEMENU_ABOUT:
			run_be_about();
			break;
		case BARNEYBAR_BEMENU_FIND:
			messenger = new BMessenger( "application/x-vnd.Be-TRAK" );
			messenger->SendMessage(message);
			break;
		case BARNEYBAR_BEMENU_PREFS:
			temp[0]="/boot/preferences";
			be_roster->Launch( "application/x-vnd.Be-TRAK", 1, temp );
			break;
		case BARNEYBAR_BEMENU_SHUTDOWN:
		case BARNEYBAR_BEMENU_REBOOT:
			messenger = new BMessenger( "application/x-vnd.Be-ROST" );
			messenger->SendMessage(message);
			break;
		default:
			BWindow::MessageReceived(message);
	}
}

void BarneyBarWindow::ScreenChanged(BRect frame, color_space mode)
{
	//(new BAlert("Info","CHANGED.", "OK"))->Go();	
	ResizeTo( frame.right, 18 );
	
	Clock->MoveTo(Bounds().right-60, 1);	
	Pager->MoveTo(Bounds().right-(count_workspaces()*16)-65, 1 );
	RightEnd->MoveTo( Bounds().right-7, 0 );
	
	
}

void BarneyBarWindow::WorkspaceActivated(int32 workspace, bool active)
{
	if ( active == true )
		Pager->CurrentWorkspace=workspace;
	Pager->Invalidate();
}
